# Generated by Django 2.2.16 on 2020-09-17 11:20
import taggit.managers
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('assets', '0002_auto_20200330_2224'),
    ]

    operations = [
        migrations.AlterField(
            model_name='asset',
            name='tags',
            field=taggit.managers.TaggableManager(help_text='Search tags are used to prefilter assets when including an asset on your object', through='taggit.TaggedItem', to='taggit.Tag', verbose_name='Tags'),
        ),
    ]
