$margin: 10px;

.asset {
	@include microfix;
	@include flex-container;
}

.asset__preview {
	border: 1px dashed $black;
	border-radius: 3px;
	background-size: 92%;
	background-repeat: no-repeat;
	background-position: 4px 4px;
	flex-grow: 0;
	margin: 0 $margin $margin 0;
	width: 100px;
	height: 100px;
}

.asset__actions {
	flex-grow: 1;
}

.asset__edit-link {
	margin-left: 10px;
}

.asset__buttons {
	width: 100%;
	padding-top: 10px;
}

.asset__select,
.asset__crop-item {
	@include flex-container;
}

.asset__select {
	margin-bottom: $margin;
	position: relative;

	input {
		position: absolute;
		// visibility: hidden;
	}

	.selectize-control {
		//width: 83%;
		flex: 1;
		margin-right: $margin;
	}
}

.asset__add-link {
	// position: absolute;
	// //left: calc(80% + 10px);
	// right: 0;
	// top: 0;
	flex: 0;
}

.asset__tags,
.asset__crop-list {
	//width: 80%;
}

.asset__crop-item {
	margin-bottom: $margin;
}

.asset__crop-label {
	display: block;
	border: $gray-light 1px solid;
	padding: 6px;
	flex-grow: 1;
}

//.asset__add-link,
.asset__crop-link {
	flex-grow: 0;
	color: $black;
	background-color: $gray-light;
	margin-right: $button-width + 10;
	font-weight: bold;

	&:hover {
		color: $black;
		background-color: $gray;
	}
}

.asset__tags {
	margin-top: $margin;
}
