.nav-dashboard {
  justify-content: center;
  flex-direction: row;
  width: 100%;
  max-width: $capped-width;
  margin-bottom: 45px;
}

.nav-dashboard__list {
  @include flex-container;
}

.nav-dashboard__item {
  margin: 0 1.25% 1% 0;
  min-width: 19%;


  // increase item width if nav length == 1 item
  &:nth-child(1):last-child {
    width: 25%;
  }

  // flex-grow if nav length < 5 items
  &:nth-child(1):nth-last-child(2),
  &:nth-child(2):last-child,

  &:nth-child(1):nth-last-child(3),
  &:nth-child(2):nth-last-child(2),
  &:nth-child(3):last-child,

  &:nth-child(1):nth-last-child(4),
  &:nth-child(2):nth-last-child(3),
  &:nth-child(3):nth-last-child(2),
  &:nth-child(4):last-child {
    flex-grow: 1;
  }

  &:nth-child(5n),
  &:last-child {
    margin-right: 0;
  }
}

.nav-dashboard__sublist {
  background-color: $white;
}

.nav-dashboard__sublist-item {
  border-top: 1px solid $bg-primary;
  padding: 20px 0;
  text-align: center;
}

.nav-dashboard__item-title {
  background-color: $white;
  display: block;
  padding: 20px 0;
  text-align: center;

  .fa {
    display: block;
    height: 45px;
    font-size: 35px;
  }
}
