%select-box {
    background-color: #fff;
    border-radius: $gridlet;
    border: 1px solid rgba($blue, 0.5);
    display: block;
    overflow: hidden;
    white-space: nowrap;
    position: relative;
    color: $blue;
    text-decoration: none;
    min-width: 200px;
    text-align: left;
}

%select-arrow {
    border-radius: 0 $gridlet - 1 $gridlet - 1 0;
    background: $light-blue;
    color: $blue;
    position: absolute;
    right: 0;
    top: 0;
    display: block;
    height: 100%;
    width: $spacer*3;

    &:after {
        color: $blue;
        content: "s";
        font: 16px/34px CompassArrows;
        height: 100%;
        width: 100%;
        text-align: center;
        position: absolute;
        top: 0;
        right: 0;
    }
}
