$DEBUG: true;

$max-width: 1400px;
$capped-width: ($max-width * 0.8);

// COLORS
$white: #fff;
$red: #ff0000;
$red-pom: #e31b23;
$red-pom-dark: #a7161b;
$black: #191919;
$gray: #7f7f7f;
$gray-light: #eee;
$gray-dark: #666;
$blue: #0084ff;
$blue-dark: #006bcd;

$bg-primary: $gray-light;

// STACKING
$z-high: 3;
$z-mid: 2;
$z-low: 1;

// LEGACY
$background-color: #eee !default;
$text-color: #333 !default;
$link-color: #f00 !default;
$dark-blue: #11232e;
$medium-blue: #8197a3;
$light-blue: #b6cad1;
$gray: #b8b8b8;
$red: #f00;

$button-width: 150px;
$width: 1280px;
$gridlet: 3px;
$spacer: $gridlet*4;
