{% extends "cms/base_popup.html" %}

{% block page_class %}{% endblock page_class %}

{% block popup_header %}
{% endblock popup_header %}

{% block content %}

<!-- NOTE: not sure if this is used by anything if it is it won't work until naming conventions are matched with 'editor' -->
 <div class="widget-insert-annotation">
  <div class="widget-editor">
    <div class="editor-toolbar">
      <div class="editor-buttons">
        <a data-wysihtml-command="bold" title="CTRL+B">b</a>
        <a data-wysihtml-command="italic" title="CTRL+I">i</a>
        <a class="spacer" data-wysihtml-command="createLink">link</a>
        <a data-wysihtml-command="undo">undo</a>
        <a class="spacer" data-wysihtml-command="redo">redo</a>
        <a class="spacer" data-wysihtml-action="change_view">view html</a>
      </div>
      <div class="editor-modals">
        <div class="modal" data-wysihtml-dialog="createLink" style="display: none;">
          <label>Enter link</label>
          <input data-wysihtml-dialog-field="href" value="http://" class="text">
          <a data-wysihtml-dialog-action="save">OK</a>
          <a data-wysihtml-dialog-action="cancel">Cancel</a>
        </div>
      </div>
    </div>
    <form>
      <textarea class="editor-textarea"></textarea>
    </form>
  </div>
  <div id="test">OK</div>
</div>

{% endblock content %}
